@@ -35,12 +35,18 @@ function get_form_action(action, params){
     return getURL(act, params);
 }
 
-var editRecord = function(id, src, target){
+var openRecord = function(id, src, target, readonly){
 
-    if (src && src != '_terp_list' && $('_terp_count').value != '0') {
-        return new One2Many(src).edit(id);
+    var kind = getNodeAttribute(src + '_set', 'kind');
+    
+    if (!kind && getElement('_o2m_' + src)) {
+        kind = "one2many";
     }
-
+        
+    if (kind == "one2many") {
+        return new One2Many(src).edit(id, readonly);
+    }
+    
     var prefix = src && src != '_terp_list' ? src + '/' : '';
 
     var model = $(prefix + '_terp_model').value;
@@ -70,65 +76,45 @@ var editRecord = function(id, src, target){
                 'limit': limit,
                 'count': count,
                 'search_domain': search_domain};
-
+                
+    var action = readonly ? 'view' : 'edit';
+        
     if (target == '_blank') {
-        return window.open(get_form_action('edit', args));
+        return window.open(get_form_action(action, args));
+    }
+    
+    if (kind == 'many2many') {
+        args['source'] = src;
+        return openWindow(get_form_action('/openm2m/edit', args));
     }
 
-    window.location.href = get_form_action('edit', args);
+    window.location.href = get_form_action(action, args);
+}
+
+var editRecord = function(id, src, target){
+    return openRecord(id, src, target, false);
+}
+
+var viewRecord = function(id, src){
+    return openRecord(id, src, null, true);
 }
 
 var editSelectedRecord = function() {
+
     var lst = new ListView('_terp_list');
     var ids = lst.getSelectedRecords();
+    
+    if (ids && ids.length > 5) {
+        var msg = _('You selected to open %(tabs)s tabs - do you want to continue?');
+        msg = msg.replace('%(tabs)s', ids.length);
+        if (!confirm(msg)) return;
+    }
 
     forEach(ids, function(id){
         editRecord(id, '_terp_list', '_blank');
     });
 }
 
-var viewRecord = function(id, src){
-
-    if (src && src != '_terp_list' && $('_terp_count').value != '0') {
-        if (getElement(src + '_set')) {
-            return editRecord(id, src);
-        } else {
-            return new One2Many(src).edit(id, true);
-        }
-    }
-
-    var prefix = src && src != '_terp_list' ? src + '/' : '';
-    var model = $(prefix + '_terp_model').value;
-    var view_ids = $(prefix + '_terp_view_ids').value;
-    var view_mode = $(prefix + '_terp_view_mode').value;
-
-    var ids = $(prefix + '_terp_ids').value;
-
-    var offset = $(prefix + '_terp_offset').value;
-    var limit = $(prefix + '_terp_limit').value;
-    var count = $(prefix + '_terp_count').value;
-
-    var domain = $(prefix + '_terp_domain').value;
-    var context = $(prefix + '_terp_context').value;
-
-    var search_domain = $('_terp_search_domain');
-    search_domain = search_domain ? search_domain.value : null;
-
-    var args = {'model': model,
-                'id': id ? id : 'False',
-                'ids': ids,
-                'view_ids': view_ids,
-                'view_mode': view_mode,
-                'domain': domain,
-                'context': context,
-                'offset': offset,
-                'limit': limit,
-                'count': count,
-                'search_domain': search_domain};
-
-    window.location.href = get_form_action('view', args);
-}
-
 var switchView = function(view_type, src){
 
     var prefix = src ? src + '/' : '';
@@ -157,12 +143,12 @@ var switch_O2M = function(view_type, src){
     
     var prefix = src ? src + '/' : '';
     var form = document.forms['view_form'];
-
+	
     var params = getFormParams();
     
     params['_terp_source'] = src;
     params['_terp_source_view_type'] = view_type;
-
+    
     if (getElement('_terp_list')){
         var ids = new ListView('_terp_list').getSelectedRecords();
         if (ids.length > 0) {
@@ -228,8 +214,6 @@ var validate_required = function(form) {
        return !el.disabled && el.id && el.name && el.id.indexOf('_terp_listfields/') == -1 && hasElementClass(el, 'requiredfield');
     }, form.elements);
 
-    log(elements.length);
-
     var result = true;
 
     for (var i=0; i<elements.length; i++){
@@ -247,8 +231,6 @@ var validate_required = function(form) {
             elem2 = MochiKit.DOM.getElement(elem.id + '_text') || elem;
         }
 
-        log(elem.name, value);
-
         if (!value) {
             addElementClass(elem2, 'errorfield');
             result = false;
@@ -256,19 +238,21 @@ var validate_required = function(form) {
             removeElementClass(elem2, 'errorfield');
         }
     }
-
-    log(1, result);
     
+    if (!result) {
+        alert(_("Invalid form, correct red fields !"));
+    }
+
     return result;
 }
 
-var submit_form = function(action, src, data, target){
+var submit_form = function(action, src, target){
     
     if (Ajax.COUNT > 0) {
-        return callLater(1, submit_form, action, src, data);
+        return callLater(1, submit_form, action, src, target);
     }
 
-    if (action == 'delete' && !confirm(_('Do you realy want to delete this record?'))) {
+    if (action == 'delete' && !confirm(_('Do you really want to delete this record?'))) {
         return false;
     }
 
@@ -278,43 +262,10 @@ var submit_form = function(action, src, data, target){
     var source = src ? (typeof(src) == "string" ? src : src.name) : null;
 
     var args = {
-        _terp_source: source,
-        _terp_data: data ? data : null
+        _terp_source: source
     };
-
-    if ((action == 'action' || action == 'relate') && $('_terp_list')){
-        var list = new ListView('_terp_list');
-        var ids = list.getSelectedRecords();
-
-        if (ids.length == 0) {
-           return alert(_('You must select at least one record.'));
-        }
-
-        args['_terp_selection'] = '[' + ids.join(',') + ']';
-    }
-
-    if ((action == 'action' || action == 'relate') && !getElement('_terp_list')){
-
-        var cur_id = getElement('_terp_id').value;
-        cur_id = parseInt(cur_id) || 0;
-
-        if (!cur_id) {
-           return alert(_('You must save this record to use the relate button!'));
-        }
-    }
-    
-    if (action == 'relate' && target == 'new') {
-        
-        args['_terp_model'] = getElement('_terp_model').value;
-        args['_terp_id'] = getElement('_terp_id').value;
-        args['_terp_domain'] = getElement('_terp_domain').value;
-        args['_terp_context'] = getElement('_terp_context').value;
-        
-        var act = get_form_action('relate', args);
-        return openWindow(act);
-    }
    
-    if (target == "new" || action == 'report' || (action == 'action' && data)){
+    if (target == "new" || target == "_blank"){
         setNodeAttribute(form, 'target', '_blank');
     }
     
@@ -380,14 +331,15 @@ var buttonClicked = function(name, btype, model, id, sure, target){
         return;
     }
 
-    params = {};
+    var params = {};
 
     params['_terp_button/name'] = name;
     params['_terp_button/btype'] = btype;
     params['_terp_button/model'] = model;
     params['_terp_button/id'] = id;
-
-    submit_form(get_form_action(btype == 'cancel' ? 'cancel' : 'save', params), null, null, target);
+    
+    var act = get_form_action(btype == 'cancel' ? 'cancel' : 'save', params);
+    submit_form(act, null, target);
 }
 
 var onBooleanClicked = function(name) {
@@ -417,22 +369,50 @@ var getFormData = function(extended) {
     fields = fields.concat(getElementsByTagAndClassName('input', null, parentNode));
     fields = fields.concat(getElementsByTagAndClassName('select', null, parentNode));
     fields = fields.concat(getElementsByTagAndClassName('textarea', null, parentNode));
-    fields = fields.concat(filter(function(e){return getNodeAttribute(e,'kind')=='picture';}, getElementsByTagAndClassName('img', null, parentNode)));
-
-    forEach(fields, function(e){
-
+    fields = fields.concat(filter(function(e){
+        return getNodeAttribute(e,'kind')=='picture';
+    }, getElementsByTagAndClassName('img', null, parentNode)));
+    
+    for(var i=0; i<fields.length; i++) {
+    
+        var e = fields[i];
+        
         if (e.tagName.toLowerCase() != 'img' && !e.name)
-            return;
+            continue;
 
         var n = e.name.replace('_terp_listfields/', '');
 
         // don't include _terp_ fields except _terp_id
         if (/_terp_/.test(n) && ! /_terp_id$/.test(n))
-            return;
+            continue;
 
         // work arround to skip o2m values (list mode)
-        if (document.getElementsByName(n + '/__id').length)
-            return;
+        if (n.indexOf('/__id') > 0) {
+        
+            n = n.replace('/__id', '');
+            
+            if ($(n + '/_terp_view_type').value == 'form') {
+                frm[n+'/__id'] = $(n+'/__id').value;
+                continue;
+            }
+            
+            // skip if editable list's editors are visible
+            if (getElementsByAttribute(['name', '^=_terp_listfields/' + n]).length) {
+                continue;
+            }
+            
+            var value = $(n + '/_terp_ids').value;
+            
+            if (extended) {
+                value = {'value': value, 
+                         'type': 'one2many', 
+                         'relation': $(n + '/_terp_model').value};
+                value = serializeJSON(value);
+            }
+            
+            frm[n] = value;
+            continue;
+        }
 
         if (extended && n.indexOf('/__id') == -1) {
 
@@ -450,20 +430,20 @@ var getFormData = function(extended) {
                 //  only the resource id and all O2M
                 n = n.replace(/_terp_id$/, '');
                 if (n && !getElement(n + '__id')) {
-                    return; 
+                    continue; 
                 }
 
                 n = n + 'id';
                 
                 if (!getElement(n)) {
-                    return;    
+                    continue;    
                 }
                 
                 kind = 'integer';
                 value = value == 'False' ? '' : value;
             }
 
-            attrs['value'] = value;
+            attrs['value'] = typeof(value) == "undefined" ? '' : value;
 
             if (kind)
                 attrs['type'] = kind;
@@ -475,22 +455,17 @@ var getFormData = function(extended) {
             if (extended > 1 && hasElementClass(e, 'requiredfield'))
                 attrs['required'] =  1;
 
-            if (value && (kind == "text" || kind == "char"))
-                attrs['value'] = '""' + value + '""';
-
             if (kind == "picture") {
-                attrs['value'] = '""' + e.src + '""';
                 n = e.id;
             }
 
-            // pythonize the attr object
-            attrs = map(function(x){return '"' + x[0] + '"' + ':' + '"' + x[1] + '"'}, items(attrs));
-            frm[n] = "{" + attrs.join(", ") + "}";
-
+            // stringify the attr object
+            frm[n] = serializeJSON(attrs);
+            
         } else {
             frm[n] = e.value;
         }
-    });
+    }
 
     return frm;
 }
@@ -531,22 +506,25 @@ var onChange = function(name) {
     var callback = getNodeAttribute(caller, 'callback');
     var change_default = getNodeAttribute(caller, 'change_default');
     
-    if (!(callback || change_default)) {
+    if (!(callback || change_default) || caller.__lock_onchange) {
         return;
     }
     
     var is_list = caller.id.indexOf('_terp_listfields') == 0;
-    var prefix =  caller.name.slice(0, caller.name.lastIndexOf('/')+1);
+    var prefix = caller.name || caller.id;
+    prefix = prefix.slice(0, prefix.lastIndexOf('/')+1);
 
     var params = getFormData(1);
     var model = is_list ? $(prefix.slice(17) + '_terp_model').value : $(prefix + '_terp_model').value;
     var context = is_list ? $(prefix.slice(17) + '_terp_context').value : $(prefix + '_terp_context').value;
+    var id = is_list ? $(prefix.slice(17) + '_terp_id').value : $(prefix + '_terp_id').value;
 
     params['_terp_caller'] = is_list ? caller.id.slice(17) : caller.id;
     params['_terp_callback'] = callback;
     params['_terp_model'] = model;
     params['_terp_context'] = context;
     params['_terp_value'] = caller.value;
+    params['id'] = id;
     
     var req = Ajax.JSON.post(callback ? '/form/on_change' : '/form/change_default_get', params);
 
@@ -556,7 +534,7 @@ var onChange = function(name) {
             return alert(obj.error);
         }
         
-        values = obj['values'];
+        values = obj['value'];
         domains = obj['domain'];
 
         domains = domains ? domains : {};
@@ -576,7 +554,10 @@ var onChange = function(name) {
             if (!fld) continue;
 
             value = values[k];
-            value = value === false || value === null ? '' : value
+            value = value === false || value === null ? '' : value;
+
+            // prevent recursive onchange
+            fld.__lock_onchange = true;
 
             if ($(prefix + k + '_id')){
                 fld = $(prefix + k + '_id');
@@ -593,25 +574,20 @@ var onChange = function(name) {
                 }
 
                 if (kind == 'many2one'){
-                    //getName(fld);
                     fld.value = value[0] || '';
                     try {
                         $(prefix + k + '_text').value = value[1] || '';
-                        ManyToOne.change_icon(fld);
                     }catch(e){}
                 }
 
-                if (kind == 'many2many'){
-                    fld.onchange();
-                }
-
                 if (kind == 'boolean') {
                     $(prefix + k + '_checkbox_').checked = value || false;
                 }
-                
-                // should be saved
-                fld.disabled = false;
+
+                MochiKit.Signal.signal(fld, 'onchange');
             }
+
+            fld.__lock_onchange = false;
         }
 
         if (obj.warning && obj.warning.message) {
@@ -667,8 +643,14 @@ function eval_domain_context_request(options){
     params['_terp_domain'] = options.domain;
     params['_terp_context'] = options.context;
     params['_terp_prefix'] = prefix;    
-    params['_terp_parent_id'] = prefix ? $(prefix + '/_terp_id').value : $('_terp_id').value;
+    params['_terp_active_id'] = prefix ? $(prefix + '/_terp_id').value : $('_terp_id').value;
+    params['_terp_active_ids'] = prefix ? $(prefix + '/_terp_ids').value : $('_terp_ids').value;
     
+    if (options.active_id) {
+        params['_terp_active_id'] = options.active_id;
+        params['_terp_active_ids'] = options.active_ids;
+    }
+        
     var parent_context = prefix ? $(prefix + '/_terp_context') : $('_terp_context');
     
     if (parent_context){
@@ -760,9 +742,11 @@ function makeContextMenu(id, kind, relation, val) {
 
             for(var r in obj.relates) {
                 var o = obj.relates[r];
-
+                
                 var a = SPAN({'class': o.action ? '' : 'disabled',
-                           'onclick': o.action ? 'hideElement(\'contextmenu\'); return ' + o.action : ''}, o.text);
+                              'onclick': o.action ? 'hideElement(\'contextmenu\'); return ' + o.action : '',
+                              'domain': o.domain,
+                              'context': o.context}, o.text);
 
                 rows = rows.concat(a);
             }
@@ -858,17 +842,7 @@ function set_as_default(field, model){
     });
 }
 
-function do_action(id, relation) {
-
-    id = $(id).value;
-
-    var act = get_form_action('action');
-    var params = {'_terp_model': relation, '_terp_id': id};
-
-    window.open(getURL(act, params));
-}
-
-function do_print(id, relation) {
+function do_report(id, relation) {
 
     id = $(id).value;
 
@@ -878,15 +852,44 @@ function do_print(id, relation) {
     window.open(getURL(act, params));
 }
 
-function do_relate(action_id, field, relation, src) {
+function do_action(action_id, field, relation, src) {
+    
+    var params = {};
+    
+    if ($('_terp_list')) {
+        var list = new ListView('_terp_list');
+        var ids = list.getSelectedRecords();
+
+        if (ids.length == 0) {
+           return alert(_('You must select at least one record.'));
+        }
+
+        params['_terp_selection'] = '[' + ids.join(',') + ']';
+    }
 
     var id = $(field).value;
-    var data = getNodeAttribute(src, 'data');
+    var domain = getNodeAttribute(src, 'domain');
+    var context = getNodeAttribute(src, 'context');
+    
+    var req = eval_domain_context_request({source: $(field).id,
+                                           active_id: id,
+                                           active_ids: params['_terp_selection'],
+                                           domain: domain, 
+                                           context: context});
+                                           
+    req.addCallback(function(obj){
+          
+        var act = get_form_action('action');
+        MochiKit.Base.update(params, {
+            '_terp_action': action_id,
+            '_terp_domain': obj.domain, 
+            '_terp_context': obj.context, 
+            '_terp_id': id,
+            '_terp_model': relation});
+        
+        window.open(getURL(act, params));
 
-    var act = get_form_action('action');
-    var params = {'_terp_data': data, '_terp_id': id, '_terp_model': relation};
-
-    window.open(getURL(act, params));
+    });
 }
 
 function on_context_menu(evt) { 
@@ -899,10 +902,11 @@ function on_context_menu(evt) {
     
     if(! kind || target.disabled)
         return;
-       
+
     var menu = $('contextmenu');
 
     if (!menu) {
+
         menu = DIV({'id': 'contextmenu', 
                     'class' : 'contextmenu', 
                     'onmouseout' : 'hideContextMenu()', 
@@ -955,7 +959,7 @@ function open_url(site){
     else
         web_site = site;
 
-    if(site.length > 0) {        
+    if(site.length > 0) {
         window.open(web_site);
     }
 }
